import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:braze_plugin/braze_plugin.dart';
import 'package:flutter/material.dart';

void main() {
  WidgetsFlutterBinding.ensureInitialized();
  final List<MethodCall> log = <MethodCall>[];
  setUpAll(() async {
    MethodChannel('braze_plugin')
        .setMockMethodCallHandler((MethodCall methodCall) async {
      log.add(methodCall);
      // If needed to mock return values:
      // switch (methodCall.method) {
      //   case 'someMethod':
      //     return someValue;
      //   default:
      //     return null;
      // }
    });
  });
  tearDown(() async {
    log.clear();
  });

  test('should call changeUser', () {
    BrazePlugin _braze = new BrazePlugin();
    String _testUser = 'thistestuser';
    _braze.changeUser(_testUser);
    expect(log, <Matcher>[
      isMethodCall(
        'changeUser',
        arguments: <String, dynamic>{
          'userId': _testUser
        },
      ),
    ]);
  });

  test('should call logContentCardClicked', () {
    BrazePlugin _braze = new BrazePlugin();
    String _data = '{"someJson":"data"}';
    BrazeContentCard _contentCard = new BrazeContentCard(_data);
    _braze.logContentCardClicked(_contentCard);
    expect(log, <Matcher>[
      isMethodCall(
        'logContentCardClicked',
        arguments: <String, dynamic>{
          'contentCardString': _contentCard.contentCardJsonString
        },
      ),
    ]);
  });

  test('should call logContentCardImpression', () {
    BrazePlugin _braze = new BrazePlugin();
    String _data = '{"someJson":"data"}';
    BrazeContentCard _contentCard = new BrazeContentCard(_data);
    _braze.logContentCardImpression(_contentCard);
    expect(log, <Matcher>[
      isMethodCall(
        'logContentCardImpression',
        arguments: <String, dynamic>{
          'contentCardString': _contentCard.contentCardJsonString
        },
      ),
    ]);
  });

  test('should call logContentCardDismissed', () {
    BrazePlugin _braze = new BrazePlugin();
    String _data = '{"someJson":"data"}';
    BrazeContentCard _contentCard = new BrazeContentCard(_data);
    _braze.logContentCardDismissed(_contentCard);
    expect(log, <Matcher>[
      isMethodCall(
        'logContentCardDismissed',
        arguments: <String, dynamic>{
          'contentCardString': _contentCard.contentCardJsonString
        },
      ),
    ]);
  });

  test('should call logInAppMessageClicked', () {
    BrazePlugin _braze = new BrazePlugin();
    String _data = '{"someJson":"data"}';
    BrazeInAppMessage _inAppMessage = new BrazeInAppMessage(_data);
    _braze.logInAppMessageClicked(_inAppMessage);
    expect(log, <Matcher>[
      isMethodCall(
        'logInAppMessageClicked',
        arguments: <String, dynamic>{
          'inAppMessageString': _inAppMessage.inAppMessageJsonString
        },
      ),
    ]);
  });

  test('should call logInAppMessageImpression', () {
    BrazePlugin _braze = new BrazePlugin();
    String _data = '{"someJson":"data"}';
    BrazeInAppMessage _inAppMessage = new BrazeInAppMessage(_data);
    _braze.logInAppMessageImpression(_inAppMessage);
    expect(log, <Matcher>[
      isMethodCall(
        'logInAppMessageImpression',
        arguments: <String, dynamic>{
          'inAppMessageString': _inAppMessage.inAppMessageJsonString
        },
      ),
    ]);
  });

  test('should call logInAppMessageButtonClicked', () {
    BrazePlugin _braze = new BrazePlugin();
    String _data = '{"someJson":"data"}';
    int _buttonId = 42;
    BrazeInAppMessage _inAppMessage = new BrazeInAppMessage(_data);
    _braze.logInAppMessageButtonClicked(_inAppMessage, _buttonId);
    expect(log, <Matcher>[
      isMethodCall(
        'logInAppMessageButtonClicked',
        arguments: <String, dynamic>{
          'inAppMessageString': _inAppMessage.inAppMessageJsonString,
          'buttonId': _buttonId
        },
      ),
    ]);
  });

  test('should call getInstallTrackingId', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.getInstallTrackingId();
    expect(log, <Matcher>[
      isMethodCall(
        'getInstallTrackingId',
        arguments: null,
      )
    ]);
  });

  test('should call addAlias', () {
    BrazePlugin _braze = new BrazePlugin();
    String _aliasName = 'someAlias';
    String _aliasLabel = 'someLabel';
    _braze.addAlias(_aliasName, _aliasLabel);
    expect(log, <Matcher>[
      isMethodCall(
        'addAlias',
        arguments: <String, dynamic>{
          'aliasName': _aliasName,
          'aliasLabel': _aliasLabel
        },
      ),
    ]);
  });

  test('should call logCustomEvent', () {
    BrazePlugin _braze = new BrazePlugin();
    String _eventName = 'someEvent';
    _braze.logCustomEvent(_eventName);
    expect(log, <Matcher>[
      isMethodCall(
        'logCustomEvent',
        arguments: <String, dynamic>{
          'eventName': _eventName
        },
      ),
    ]);
  });

  test('should call logCustomEventWithProperties', () {
    BrazePlugin _braze = new BrazePlugin();
    String _eventName = 'someEvent';
    Map<String, dynamic> _properties = {'someKey': 'someValue'};
    _braze.logCustomEventWithProperties(_eventName, _properties);
    expect(log, <Matcher>[
      isMethodCall(
        'logCustomEvent',
        arguments: <String, dynamic>{
          'eventName': _eventName,
          'properties': _properties
        },
      ),
    ]);
  });

  test('should call logPurchase', () {
    BrazePlugin _braze = new BrazePlugin();
    String _productId = 'someProduct';
    String _currencyCode = 'someCurrencyCode';
    double _price = 4.2;
    int _quantity = 42;
    _braze.logPurchase(_productId, _currencyCode, _price, _quantity);
    expect(log, <Matcher>[
      isMethodCall(
        'logPurchase',
        arguments: <String, dynamic>{
          'productId': _productId,
          'currencyCode': _currencyCode,
          'price': _price,
          'quantity': _quantity
        },
      ),
    ]);
  });

  test('should call logPurchaseWithProperties', () {
    BrazePlugin _braze = new BrazePlugin();
    String _productId = 'someProduct';
    String _currencyCode = 'someCurrencyCode';
    double _price = 4.2;
    int _quantity = 42;
    Map<String, dynamic> _properties = {'someKey': 'someValue'};
    _braze.logPurchaseWithProperties(_productId, _currencyCode, _price, _quantity, _properties);
    expect(log, <Matcher>[
      isMethodCall(
        'logPurchase',
        arguments: <String, dynamic>{
          'productId': _productId,
          'currencyCode': _currencyCode,
          'price': _price,
          'quantity': _quantity,
          'properties': _properties
        },
      ),
    ]);
  });

  test('should call addToCustomAttributeArray', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    String _value = 'someValue';
    _braze.addToCustomAttributeArray(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'addToCustomAttributeArray',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value
        },
      ),
    ]);
  });

  test('should call removeFromCustomAttributeArray', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    String _value = 'someValue';
    _braze.removeFromCustomAttributeArray(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'removeFromCustomAttributeArray',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value
        },
      ),
    ]);
  });

  test('should call setStringCustomUserAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    String _value = 'someValue';
    _braze.setStringCustomUserAttribute(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'setStringCustomUserAttribute',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value
        },
      ),
    ]);
  });

  test('should call setDoubleCustomUserAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    double _value = 4.2;
    _braze.setDoubleCustomUserAttribute(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'setDoubleCustomUserAttribute',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value
        },
      ),
    ]);
  });

  test('should call setBoolCustomUserAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    bool _value = false;
    _braze.setBoolCustomUserAttribute(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'setBoolCustomUserAttribute',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value
        },
      ),
    ]);
  });

  test('should call setIntCustomUserAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    int _value = 42;
    _braze.setIntCustomUserAttribute(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'setIntCustomUserAttribute',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value
        },
      ),
    ]);
  });

  test('should call incrementCustomUserAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    int _value = 42;
    _braze.incrementCustomUserAttribute(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'incrementCustomUserAttribute',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value
        },
      ),
    ]);
  });

  test('should call setLocationCustomAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    double _lat = 12.34;
    double _long = 56.78;
    _braze.setLocationCustomAttribute(_key, _lat, _long);
    expect(log, <Matcher>[
      isMethodCall(
        'setLocationCustomAttribute',
        arguments: <String, dynamic>{
          'key': _key,
          'lat': _lat,
          'long': _long
        },
      ),
    ]);
  });

  test('should call setDateCustomUserAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    DateTime _value = new DateTime.now();
    _braze.setDateCustomUserAttribute(_key, _value);
    expect(log, <Matcher>[
      isMethodCall(
        'setDateCustomUserAttribute',
        arguments: <String, dynamic>{
          'key': _key,
          'value': _value.millisecondsSinceEpoch ~/ 1000
        },
      ),
    ]);
  });

  test('should call unsetCustomUserAttribute', () {
    BrazePlugin _braze = new BrazePlugin();
    String _key = 'someKey';
    _braze.unsetCustomUserAttribute(_key);
    expect(log, <Matcher>[
      isMethodCall(
        'unsetCustomUserAttribute',
        arguments: <String, dynamic>{
          'key': _key
        },
      ),
    ]);
  });

  test('should call setFirstName', () {
    BrazePlugin _braze = new BrazePlugin();
    String _firstName = 'someFirstName';
    _braze.setFirstName(_firstName);
    expect(log, <Matcher>[
      isMethodCall(
        'setFirstName',
        arguments: <String, dynamic>{
          'firstName': _firstName
        },
      ),
    ]);
  });

  test('should call setLastName', () {
    BrazePlugin _braze = new BrazePlugin();
    String _lastName = 'someLastName';
    _braze.setLastName(_lastName);
    expect(log, <Matcher>[
      isMethodCall(
        'setLastName',
        arguments: <String, dynamic>{
          'lastName': _lastName
        },
      ),
    ]);
  });

  test('should call setEmail', () {
    BrazePlugin _braze = new BrazePlugin();
    String _email = 'someEmail';
    _braze.setEmail(_email);
    expect(log, <Matcher>[
      isMethodCall(
        'setEmail',
        arguments: <String, dynamic>{
          'email': _email
        },
      ),
    ]);
  });

  test('should call setDateOfBirth', () {
    BrazePlugin _braze = new BrazePlugin();
    int _year = 2000;
    int _month = 1;
    int _day = 22;
    _braze.setDateOfBirth(_year, _month, _day);
    expect(log, <Matcher>[
      isMethodCall(
        'setDateOfBirth',
        arguments: <String, dynamic>{
          'year': _year,
          'month': _month,
          'day': _day
        },
      ),
    ]);
  });

  test('should call setGender', () {
    BrazePlugin _braze = new BrazePlugin();
    String _gender = 'f';
    _braze.setGender(_gender);
    expect(log, <Matcher>[
      isMethodCall(
        'setGender',
        arguments: <String, dynamic>{
          'gender': _gender
        },
      ),
    ]);
  });

  test('should call setLanguage', () {
    BrazePlugin _braze = new BrazePlugin();
    String _language = 'es';
    _braze.setLanguage(_language);
    expect(log, <Matcher>[
      isMethodCall(
        'setLanguage',
        arguments: <String, dynamic>{
          'language': _language
        },
      ),
    ]);
  });

  test('should call setCountry', () {
    BrazePlugin _braze = new BrazePlugin();
    String _country = 'JP';
    _braze.setCountry(_country);
    expect(log, <Matcher>[
      isMethodCall(
        'setCountry',
        arguments: <String, dynamic>{
          'country': _country
        },
      ),
    ]);
  });

  test('should call setHomeCity', () {
    BrazePlugin _braze = new BrazePlugin();
    String _homeCity = 'someHomeCity';
    _braze.setHomeCity(_homeCity);
    expect(log, <Matcher>[
      isMethodCall(
        'setHomeCity',
        arguments: <String, dynamic>{
          'homeCity': _homeCity
        },
      ),
    ]);
  });

  test('should call setPhoneNumber', () {
    BrazePlugin _braze = new BrazePlugin();
    String _phoneNumber = '8675309';
    _braze.setPhoneNumber(_phoneNumber);
    expect(log, <Matcher>[
      isMethodCall(
        'setPhoneNumber',
        arguments: <String, dynamic>{
          'phoneNumber': _phoneNumber
        },
      ),
    ]);
  });

  test('should call setAvatarImageUrl', () {
    BrazePlugin _braze = new BrazePlugin();
    String _avatarImageUrl = 'https://raw.githubusercontent.com/Appboy/appboy-react-sdk/master/braze-logo.png';
    _braze.setAvatarImageUrl(_avatarImageUrl);
    expect(log, <Matcher>[
      isMethodCall(
        'setAvatarImageUrl',
        arguments: <String, dynamic>{
          'avatarImageUrl': _avatarImageUrl
        },
      ),
    ]);
  });

  test('should call setAttributionData', () {
    BrazePlugin _braze = new BrazePlugin();
    String _network = 'someNetwork';
    String _campaign = 'someCampaign';
    String _adGroup = 'someAdGroup';
    String _creative = 'someCreative';
    _braze.setAttributionData(_network, _campaign, _adGroup, _creative);
    expect(log, <Matcher>[
      isMethodCall(
        'setAttributionData',
        arguments: <String, dynamic>{
          'network': _network,
          'campaign': _campaign,
          'adGroup': _adGroup,
          'creative': _creative 
        },
      ),
    ]);
  });

  test('should call registerAndroidPushToken', () {
    BrazePlugin _braze = new BrazePlugin();
    String _pushToken = 'someToken';
    _braze.registerAndroidPushToken(_pushToken);
    expect(log, <Matcher>[
      isMethodCall(
        'registerAndroidPushToken',
        arguments: <String, dynamic>{
          'pushToken': _pushToken
        },
      ),
    ]);
  });

  test('should call requestImmediateDataFlush', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.requestImmediateDataFlush();
    expect(log, <Matcher>[
      isMethodCall(
        'requestImmediateDataFlush',
        arguments: null,
      ),
    ]);
  });

  test('should call wipeData', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.wipeData();
    expect(log, <Matcher>[
      isMethodCall(
        'wipeData',
        arguments: null,
      ),
    ]);
  });

  test('should call requestContentCardsRefresh', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.requestContentCardsRefresh();
    expect(log, <Matcher>[
      isMethodCall(
        'requestContentCardsRefresh',
        arguments: null,
      ),
    ]);
  });

  test('should call launchContentCards', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.launchContentCards();
    expect(log, <Matcher>[
      isMethodCall(
        'launchContentCards',
        arguments: null,
      ),
    ]);
  });

  test('should call requestLocationInitialization', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.requestLocationInitialization();
    expect(log, <Matcher>[
      isMethodCall(
        'requestLocationInitialization',
        arguments: null,
      ),
    ]);
  });

  test('should call enableSDK', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.enableSDK();
    expect(log, <Matcher>[
      isMethodCall(
        'enableSDK',
        arguments: null,
      ),
    ]);
  });

  test('should call disableSDK', () {
    BrazePlugin _braze = new BrazePlugin();
    _braze.disableSDK();
    expect(log, <Matcher>[
      isMethodCall(
        'disableSDK',
        arguments: null,
      ),
    ]);
  });

  test('should call setPushNotificationSubscriptionType', () {
    BrazePlugin _braze = new BrazePlugin();
    SubscriptionType _type = SubscriptionType.opted_in;
    _braze.setPushNotificationSubscriptionType(_type);
    expect(log, <Matcher>[
      isMethodCall(
        'setPushNotificationSubscriptionType',
        arguments: <String, dynamic>{
          'type': _type.toString()
        },
      ),
    ]);
  });

  test('should call setEmailNotificationSubscriptionType', () {
    BrazePlugin _braze = new BrazePlugin();
    SubscriptionType _type = SubscriptionType.opted_in;
    _braze.setEmailNotificationSubscriptionType(_type);
    expect(log, <Matcher>[
      isMethodCall(
        'setEmailNotificationSubscriptionType',
        arguments: <String, dynamic>{
          'type': _type.toString()
        },
      ),
    ]);
  });
}